#this import statement allows access to the karamba functions
import karamba
import re

#This the name of the plugin wich will be displayed in the configuration GUI
plugin_name = "Amarok AIO"
#This is the file name of the plugin's icon on wich the user will click to open it
icon = 'amarok.png'
#This is the 'dict' of configurable parameters of the plugin with their default value,
#they have to be edited in the configuration_gui and parameters name must be in the form:
#    "smallword_parameter'sName" where smallword is an abbreviation of the name of the plugin
default_param = {}

#This function is called in the initWidget(widget) function of the theme
#so what is typed here will be executed at once
def init(widget):
	pass

def searchSpace(text, index):
	notfound = True
	i = index
	while ( notfound and i != 0 ):
		notfound = not( text[i] == " " )
		i = i - 1
	return (i + 1)

#This class is called when the plugin's icon is clicked, it opens the page of the plugin
#It must have "self.title" wich is the title of the page,
#self.line wich is a small line under the title and all the functions already defined
#Arguments: theme is the path to current theme (string)
#           color is the color of all texts excepted the title in the form of a list [int red, int green, int blue]
#           titleColor : you know what it is
#           conf is the configuration 'dict' you can access your parameters like that conf["smallword_parameter'sName"]
class Plugin:
	
	def __init__(self, widget, theme, color, titleColor, conf):
		self.title = karamba.createText(widget, 36, 34, 200, 20, "Amarok control")
		karamba.changeTextSize(widget, self.title, 20)
		self.line = karamba.createImage(widget, 31, 58, theme + "line.png")
		self.time_pid, self.total_pid, self.title_pid, self.cover_pid, self.artist_pid = None, None, None, None, None
		self.time = 0
		self.updated = 0
		self.total = 0
		self.coverState = 0
		self.artist = ""
		self.title_str = ""
		self.scrollTitle = [False, 0]
		self.artist_y = 60
		self.title_y = 80
		self.coverImg = karamba.createImage(widget, 48, 73, "")
		self.boxImg = karamba.createImage(widget, 34, 70, conf["img_dir"] + "amarok/cd.png")
		self.bar = karamba.createBar(widget, 112, 122, 136, 6, theme + "bar_140.png")
		self.cover = karamba.createImage(widget, 110, 120, theme + "border_140.png")
		
		self.artistText = karamba.createText(widget, 210, 60, 220, 29, "")
		self.titleText = karamba.createText(widget, 220, 80, 210, 23, "")
		self.timeText = karamba.createText(widget, 190, 134, 100, 14, " / ")
		karamba.setTextAlign(widget, self.artistText, "CENTER")
		karamba.setTextAlign(widget, self.titleText, "CENTER")
		karamba.setTextAlign(widget, self.timeText, "CENTER")
		
		self.prevImg = karamba.createImage(widget, 0, 0, conf["img_dir"] + "amarok/prev.png")
		self.playImg = karamba.createImage(widget, 0, 0, conf["img_dir"] + "amarok/play.png")
		self.pausImg = karamba.createImage(widget, 0, 0, conf["img_dir"] + "amarok/pause.png")
		self.stopImg = karamba.createImage(widget, 0, 0, conf["img_dir"] + "amarok/stop.png")
		self.nextImg = karamba.createImage(widget, 0, 0, conf["img_dir"] + "amarok/next.png")
		
		karamba.attachClickArea(widget, self.prevImg, "dcop amarok player prev")
		karamba.attachClickArea(widget, self.playImg, "dcop amarok player play")
		karamba.attachClickArea(widget, self.pausImg, "dcop amarok player pause")
		karamba.attachClickArea(widget, self.stopImg, "dcop amarok player stop")
		karamba.attachClickArea(widget, self.nextImg, "dcop amarok player next")
		
		self.image_list = [self.coverImg, self.boxImg, self.prevImg, self.playImg, self.pausImg, self.stopImg, self.nextImg, self.cover]
		self.text_list = [self.artistText, self.titleText, self.timeText]
		
		
		self.updateCover(widget, "nocover")
		self.update(widget)
	
	#This function is used to set shadows
	def setTextShadow(self, widget, bool):
		for text in self.text_list:
			karamba.changeTextShadow(widget, text, int(bool))
	
	def setTextColor(self, widget, color):
		for text in self.text_list:
			karamba.changeTextColor(widget, text, color[0], color[1], color[2])
	
	def setTextFont(self, widget, font):
		for text in self.text_list:
			karamba.changeTextFont(widget, text, font)
	
	#This function is called in the widgetUpdated(widget) function of the theme
	#which is called every seconds
	def update(self, widget):
		self.updated = 0
		time_command = ["dcop", "amarok", "player", "trackCurrentTime"]
		self.time_pid = karamba.executeInteractive(widget, time_command)
		
		if self.scrollTitle[0]:
			index = self.scrollTitle[1]
			lenght = len(self.title_str)
			if (index >= lenght):
				index = index - lenght
				self.scrollTitle[1] = index
			if (self.coverState == 1):
				if (index + 21 <= lenght):
					karamba.changeText(widget, self.titleText, self.title_str[index:index+20])
				else:
					rest = lenght - index
					karamba.changeText(widget, self.titleText, self.title_str[index:index+rest] + self.title_str[0:21-rest])
			elif (self.coverState == 0):
				if (index + 30 <= lenght):
					karamba.changeText(widget, self.titleText, self.title_str[index:index+29])
				else:
					rest = lenght - index
					karamba.changeText(widget, self.titleText, self.title_str[index:index+rest] + self.title_str[0:30-rest])
			self.scrollTitle[1] = index + 2
	
	#This function is called in the commandOutput(widget, pid, output) function of the theme
	def commandOutput(self, widget, pid, output):
		
		if ( pid == self.time_pid ):
			temp = int(output)
			if ( self.total != 0 ):
				percent = 100 * self.time / self.total
				karamba.setBarValue(widget, self.bar, percent)
			mins = temp/60
			secs = temp%60
			temp2 = karamba.getTextValue(widget, self.timeText)
			i = temp2.find(" / ")
			if ( secs < 10 ):
				text = str(mins) + ":0" + str(secs) + temp2[i:]
			else:
				text = str(mins) + ":" + str(secs) + temp2[i:]
			karamba.changeText(widget, self.timeText, text)
			#this checks if the widget needs an update due to a current time change
			if not( temp == self.time + 1 or  temp == self.time ):
				total_command = ["dcop", "amarok", "player", "trackTotalTime"]
				self.total_pid = karamba.executeInteractive(widget, total_command)
# 				print "!! currentTime has changed: checking if it needs a complete update !!"
			self.time = temp
			
		if ( pid == self.total_pid ):
			temp = int(output)
			if ( temp != self.total ):
				self.updated = 1
				self.total = temp
				mins = self.total/60
				secs = self.total%60
				if ( secs < 10 ):
					karamba.changeText(widget, self.timeText, "0:00 / " + str(mins) + ":0" + str(secs))
				else:
					karamba.changeText(widget, self.timeText, "0:00 / " + str(mins) + ":" + str(secs))
				cover_command = ["dcop", "amarok", "player", "coverImage"]
				self.cover_pid = karamba.executeInteractive(widget, cover_command)
			title_command = ["dcop", "amarok", "player", "title"]
			self.title_pid = karamba.executeInteractive(widget, title_command)
				
		
		if ( pid == self.title_pid ):
			temp = output.replace('\n','')
			if ( temp != self.title_str ):
				self.title_str = temp
				if ( self.updated == 0 ):
					cover_command = ["dcop", "amarok", "player", "coverImage"]
					self.cover_pid = karamba.executeInteractive(widget, cover_command)
				else:
					self.updateTitle(widget, self.title_str)
		
		if ( pid == self.cover_pid ):
			self.updateCover(widget, output.replace('\n',''))
			artist_command = ["dcop", "amarok", "player", "artist"]
			self.artist_pid = karamba.executeInteractive(widget, artist_command)
			self.updateTitle(widget, self.title_str)
		
		if ( pid == self.artist_pid ):
			self.updateArtist(widget, output.replace('\n',''))
	
	#This function is called in the meterClicked(widget, meter, button) function of the theme
	def meterClicked(self, widget, meter, button):
		pass
	
	#This function is called when another page is to be opened, 
	#all the superkaramba widgets you have created have to be deleted here
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		karamba.deleteBar(widget, self.bar)
		for image in self.image_list:
			karamba.deleteImage(widget, image)
		for text in self.text_list:
			karamba.deleteText(widget, text)
	
	def updateCover(self, widget, path):
		match = re.search("nocover", path)
		if ( match == None ):
			self.coverState = 1
			karamba.showImage(widget, self.coverImg)
			karamba.showImage(widget, self.boxImg)
			karamba.setImagePath(widget, self.coverImg, path)
			karamba.resizeImage(widget, self.coverImg, 100, 100)
			karamba.moveImage(widget, self.prevImg, 183, 146)
			karamba.moveImage(widget, self.nextImg, 266, 146)
			karamba.moveImage(widget, self.playImg, 214, 150)
			karamba.moveImage(widget, self.pausImg, 183, 165)
			karamba.moveImage(widget, self.stopImg, 266, 165)
			karamba.moveBar(widget, self.bar, 172, 125)
			karamba.moveImage(widget, self.cover, 170, 123)
			karamba.moveText(widget, self.timeText, 190, 135)
		else :
			self.coverState = 0
			karamba.hideImage(widget, self.coverImg)
			karamba.hideImage(widget, self.boxImg)
			karamba.moveImage(widget, self.prevImg, 90, 160)
			karamba.moveImage(widget, self.pausImg, 122, 160)
			karamba.moveImage(widget, self.playImg, 154, 154)
			karamba.moveImage(widget, self.stopImg, 208, 160)
			karamba.moveImage(widget, self.nextImg, 240, 160)
			karamba.moveBar(widget, self.bar, 112, 129)
			karamba.moveImage(widget, self.cover, 110, 127)
			karamba.moveText(widget, self.timeText, 130, 139)
	
	def updateArtist(self, widget, text):
		lenght = len(text)
		if ( self.coverState == 1 ):
			karamba.resizeText(widget, self.artistText, 160, 30)
			if (lenght > 17):
				i = searchSpace(text, 18)
				sub = text[i+1:lenght]
				text = text[0:i] + "\n" + sub
				self.artist_y = 63
				self.title_y = 102
			else:
				self.artist_y = 73
				self.title_y = 93
			karamba.moveText(widget, self.artistText, 160, self.artist_y)
			karamba.moveText(widget, self.titleText, 160, self.title_y)
			karamba.changeTextSize(widget, self.artistText, 16)
			karamba.changeText(widget, self.artistText, text)
			
		if ( self.coverState == 0 ):
			karamba.resizeText(widget, self.artistText, 280, 30)
			if (lenght > 28):
				i = searchSpace(text, 29)
				sub = text[i+1:lenght]
				text = text[0:i] + "\n" + sub
				self.artist_y = 61
				self.title_y = 104
			else:
				self.artist_y = 69
				self.title_y = 95
			karamba.moveText(widget, self.artistText, 40, self.artist_y)
			karamba.moveText(widget, self.titleText, 40, self.title_y)
			karamba.changeTextSize(widget, self.artistText, 18)
			karamba.changeText(widget, self.artistText, text)
	
	
	def updateTitle(self, widget, text):
		lenght = len(text)
		self.title_str = text
		if ( self.coverState == 1 ):
			karamba.resizeText(widget, self.titleText, 160, 30)
			karamba.changeTextSize(widget, self.titleText, 14)
			if (lenght > 20):
				self.scrollTitle = [True, 0]
				self.title_str = text + " - "
			else:
				self.scrollTitle = [False, 0]
			karamba.changeText(widget, self.titleText, text)
		
		if ( self.coverState == 0 ):
			karamba.resizeText(widget, self.titleText, 280, 30)
			karamba.changeTextSize(widget, self.titleText, 16)
			if (lenght > 29):
				self.scrollTitle = [True, 0]
				self.title_str = text + " - "
			else:
				self.scrollTitle = [False, 0]
			karamba.changeText(widget, self.titleText, text)
